@@ -41,3 +41,13 @@ setClass(
 #' @keywords internal
 #'
 setClassUnion("char_null", c("character", "NULL"))
+
+
+#' samc mat_null class
+#'
+#' Class for grouping matrix and NULL data types
+#'
+#' @name mat_null-class
+#' @keywords internal
+#'
+setClassUnion("mat_null", c("matrix", "NULL"))
